<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

add_action( 'widgets_init', function(){
     register_widget( 'NNRobots_Popular_Posts_Widget' );
});

/*
 * AJAX hooks
 */

add_action( 'wp_ajax_nnr_display_popular_posts', 			array('NNRobots_Popular_Posts_Widget', 'ajax_display_popular_posts'));
add_action( 'wp_ajax_nopriv_nnr_display_popular_posts', 	array('NNRobots_Popular_Posts_Widget', 'ajax_display_popular_posts'));
add_action( 'init', 										array('NNRobots_Popular_Posts_Widget', 'load_textdomain'));

/**
 * Adds NNRobots_Popular_Posts_Widget widget.
 */
class NNRobots_Popular_Posts_Widget extends WP_Widget {

	/**
	 * text_domain
	 *
	 * (default value: 'nnrobots-popular-posts')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $text_domain = 'nnrobots-popular-posts';

	/**
	 * prefix
	 *
	 * (default value: 'nnrobots_popular_posts_widget_')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $prefix = 'nnrobots_popular_posts_widget_';

	/**
	 * prefix_dash
	 *
	 * (default value: 'nnr-pp-')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $prefix_dash = 'nnr-pp-';

	/**
	 * Load the text domain
	 *
	 * @since 1.0.0
	 */
	static function load_textdomain() {
		load_plugin_textdomain(self::$text_domain, false, NNROBOTS_POPULAR_POSTS_PLUGIN_DIR . '/languages');
	}

	/**
	 * Register widget with WordPress.
	 */
	function __construct() {
		parent::__construct(
			self::$prefix_dash . 'widget', // Base ID
			__('Popular Posts Pro', self::$text_domain), // Name
			array( 'description' => __( 'Display your most popular posts', self::$text_domain ), ) // Args
		);
	}

	/**
	 * Display all content using AJAX as to no effect page load times
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	public static function ajax_display_popular_posts() {

		if (isset($_POST['shortcode']) && $_POST['shortcode']) {
			echo json_encode(array('content' => self::display_popular_posts($_POST['instance'], true), 'widget_id' => $_POST['widget_id']));
		} else {
			echo json_encode(array('content' => self::display_popular_posts($_POST['instance']), 'widget_id' => $_POST['widget_id']));
		}

		die();
	}

	/**
	 * Front-end display of widget.
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args, $instance ) {

		$title = apply_filters( 'widget_title', $instance['title'] );

		echo $args['before_widget'];

		if ( ! empty( $title ) ) {
			echo $args['before_title'] . $title . $args['after_title'];
		}

		list($r, $g, $b)                               = sscanf($instance['color'], "#%02x%02x%02x");
		list($text_r, $text_g, $text_b)                = sscanf($instance['text_color'], "#%02x%02x%02x");
		list($nl_r, $nl_g, $nl_b)                      = sscanf($instance['numbered_list_color'], "#%02x%02x%02x");
		list($nl_circle_r, $nl_circle_g, $nl_circle_b) = sscanf($instance['numbered_list_circle_color'], "#%02x%02x%02x");

		if (isset($instance['async']) && $instance['async']) {
			echo '<script type="text/javascript">
			jQuery(document).ready(function($) {

				var data = {
					"action": "nnr_display_popular_posts",
					"instance" : ' . json_encode($instance) . ',
					"widget_id" : "' . $this->id . '"
				};

				$.post("' . admin_url( 'admin-ajax.php' ) . '", data, function(response) {
					response = jQuery.parseJSON(response);
					$("#" + response.widget_id).append(response.content).hide().fadeIn();';

					if ($instance['layout'] == 'masonry') {
						echo '$("#" + response.widget_id + " .' . self::$prefix_dash . 'masonry").justifiedGallery({
							lastRow: "justify",
							rowHeight: 75,
						});';
					}

					if ($instance['layout'] == 'slider') {
						echo '$("#" + response.widget_id + " .' . self::$prefix_dash . 'slider").flexslider({
							animation: "fade",';

						if (isset($instance[ 'show_nav' ]) && $instance[ 'show_nav' ]) {
							echo 'directionNav: true,';
						} else {
							echo 'directionNav: false,';
						}

						echo '});';
					}

				echo '});
			});
			</script>';
		} else {
			echo self::display_popular_posts($instance);
			echo '<script type="text/javascript">
				jQuery(document).ready(function($) {';

					if ($instance['layout'] == 'masonry') {
						echo '$("#" + response.widget_id + " .' . self::$prefix_dash . 'masonry").justifiedGallery({
							lastRow: "justify",
							rowHeight: 75,
						});';
					}

					if ($instance['layout'] == 'slider') {
						echo '$("#" + response.widget_id + " .' . self::$prefix_dash . 'slider").flexslider({
							animation: "slide",';


						if (isset($instance[ 'show_nav' ]) && $instance[ 'show_nav' ]) {
							echo 'directionNav: true,';
						} else {
							echo 'directionNav: false,';
						}

						echo '});';
					}

				echo ';});
			</script>';
		}

		if ($instance['style_override']) {

			?><style>

				/* Background Color */

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>bar-graph .<?php echo self::$prefix_dash; ?>bar-graph-color {
					background-color: rgba(<?php echo $r; ?>, <?php echo $g; ?>, <?php echo $b; ?>, 0.5);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>line-graph .<?php echo self::$prefix_dash; ?>line-graph-color {
					background-color: rgba(<?php echo $r; ?>, <?php echo $g; ?>, <?php echo $b; ?>, 1);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tile-item {
					background-color: rgba(<?php echo $r; ?>, <?php echo $g; ?>, <?php echo $b; ?>, 0.8);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>title {
					background-color: rgba(<?php echo $r; ?>, <?php echo $g; ?>, <?php echo $b; ?>, 0.9);
				}

				/* Text Color */

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>list li a {
					color: rgb(<?php echo $text_r; ?>, <?php echo $text_g; ?>, <?php echo $text_b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>list li span {
					color: rgb(<?php echo $text_r; ?>, <?php echo $text_g; ?>, <?php echo $text_b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>thumbnail li a {
					color: rgb(<?php echo $text_r; ?>, <?php echo $text_g; ?>, <?php echo $text_b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>thumbnail li span {
					color: rgb(<?php echo $text_r; ?>, <?php echo $text_g; ?>, <?php echo $text_b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>bar-graph-color a {
					color: rgb(<?php echo $text_r; ?>, <?php echo $text_g; ?>, <?php echo $text_b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>bar-graph_color span {
					color: rgb(<?php echo $text_r; ?>, <?php echo $text_g; ?>, <?php echo $text_b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>line-graph-content a {
					color: rgb(<?php echo $text_r; ?>, <?php echo $text_g; ?>, <?php echo $text_b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>line-graph-content span {
					color: rgb(<?php echo $text_r; ?>, <?php echo $text_g; ?>, <?php echo $text_b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>title {
					color: rgb(<?php echo $text_r; ?>, <?php echo $text_g; ?>, <?php echo $text_b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>stats *,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>meta-data *  {
					color: rgb(<?php echo $text_r; ?>, <?php echo $text_g; ?>, <?php echo $text_b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>fancy-colors-content a,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>fancy-colors-content .<?php echo self::$prefix_dash; ?>stats,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>fancy-colors-content .<?php echo self::$prefix_dash; ?>meta-data {
					color: rgb(<?php echo $text_r; ?>, <?php echo $text_g; ?>, <?php echo $text_b; ?>);
				}

				/* Numbered List */

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>numbered-list-big li > div:first-of-type:before {
					color: rgb(<?php echo $nl_r; ?>, <?php echo $nl_g; ?>, <?php echo $nl_b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>numbered-list-circle li > div:first-of-type:before {
					color: rgb(<?php echo $nl_r; ?>, <?php echo $nl_g; ?>, <?php echo $nl_b; ?>);
					background-color: rgb(<?php echo $nl_circle_r; ?>, <?php echo $nl_circle_g; ?>, <?php echo $nl_circle_b; ?>);
				}

			</style><?php
		} else {
			?><style>

				/* Background Color */

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>bar-graph .<?php echo self::$prefix_dash; ?>bar-graph-color {
					background-color: rgba(190, 190, 190, 0.5);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>line-graph .<?php echo self::$prefix_dash; ?>line-graph-color {
					background-color: rgba(0, 0, 0, 1);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tile-item {
					background-color: rgba(0, 0, 0, 0.8);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>title {
					background-color: rgba(0, 0, 0, 0.9);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>title {
					color: white;
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>stats *,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>meta-data *  {
					color: white;
				}

			</style><?php
		}

		// Include masonry files

		if ($instance['layout'] == 'masonry') {
			wp_enqueue_script(self::$prefix_dash . 'justified_gallery_js', plugins_url('core/includes/masonry/jquery.justifiedGallery.min.js', __FILE__), array('jquery'));
			wp_enqueue_style(self::$prefix_dash . 'justified_gallery_css', plugins_url('core/includes/masonry/justifiedGallery.min.css', __FILE__));
		}

		// Include slider files

		if ($instance['layout'] == 'slider') {
			wp_enqueue_script(self::$prefix_dash . 'slider_js', plugins_url('core/includes/flex-slider/jquery.flexslider-min.js', __FILE__), array('jquery'));
			wp_enqueue_style(self::$prefix_dash . 'slider_css', plugins_url('core/includes/flex-slider/flexslider.css', __FILE__));
		}

		wp_enqueue_style(self::$prefix_dash . 'widget_css', plugins_url('core/css/nnr-popular-posts.css', __FILE__));

		echo $args['after_widget'];
	}

	/**
	 * Back-end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {

		// Title

		if ( isset( $instance[ 'title' ] ) ) {
			$title = $instance[ 'title' ];
		} else {
			$title = __('Popular Posts', self::$text_domain );
		}

		// Content

		if ( isset( $instance[ 'content' ] ) ) {
			$content = $instance[ 'content' ];
		} else {
			$content = __('only_posts', self::$text_domain );
		}

		// Content posts in category

		if ( isset( $instance[ 'content_post_category' ] ) ) {
			$content_post_category = $instance[ 'content_post_category' ];
		} else {
			$content_post_category = __('', self::$text_domain );
		}

		// Content posts in tag

		if ( isset( $instance[ 'content_post_tag' ] ) ) {
			$content_post_tag = $instance[ 'content_post_tag' ];
		} else {
			$content_post_tag = __('', self::$text_domain );
		}

		// Content custom post type

		if ( isset( $instance[ 'content_custom_post_type' ] ) ) {
			$content_custom_post_type = $instance[ 'content_custom_post_type' ];
		} else {
			$content_custom_post_type = __('', self::$text_domain );
		}

		// Popularity

		if ( isset( $instance[ 'popularity' ] ) ) {
			$popularity = $instance[ 'popularity' ];
		} else {
			$popularity = __('views', self::$text_domain );
		}

		// Popularity Time Range

		if ( isset( $instance[ 'popularity_time_range' ] ) ) {
			$popularity_time_range = $instance[ 'popularity_time_range' ];
		} else {
			$popularity_time_range = __('all_time', self::$text_domain );
		}

		// Layout

		if ( isset( $instance[ 'layout' ] ) ) {
			$layout = $instance[ 'layout' ];
		} else {
			$layout = __('list', self::$text_domain );
		}

		// Color

		if ( isset( $instance[ 'color' ] ) ) {
			$color = $instance[ 'color' ];
		} else {
			$color = __('#d3d3d3', self::$text_domain );
		}

		// Fancy Colors

		if ( isset( $instance[ 'fancy_colors' ] ) ) {
			$fancy_colors = $instance[ 'fancy_colors' ];
		} else {
			$fancy_colors = __('#D14841,#F37934,#FAC51C,#61BD6D,#2C82C9,#553982', self::$text_domain );
		}

		// Text Color

		if ( isset( $instance[ 'text_color' ] ) ) {
			$text_color = $instance[ 'text_color' ];
		} else {
			$text_color = __('#000000', self::$text_domain );
		}

		// Num of Posts

		if ( isset( $instance[ 'num_of_posts' ] ) ) {
			$num_of_posts = $instance[ 'num_of_posts' ];
		} else {
			$num_of_posts = __('5', self::$text_domain );
		}

		// Numbered List Style

		if ( isset( $instance[ 'numbered_list_style' ] ) ) {
			$numbered_list_style = $instance[ 'numbered_list_style' ];
		} else {
			$numbered_list_style = 'normal';
		}

		// Numbered List Color

		if ( isset( $instance[ 'numbered_list_color' ] ) ) {
			$numbered_list_color = $instance[ 'numbered_list_color' ];
		} else {
			$numbered_list_color = '#ffffff';
		}

		// Numbered List Circle Color

		if ( isset( $instance[ 'numbered_list_circle_color' ] ) ) {
			$numbered_list_circle_color = $instance[ 'numbered_list_circle_color' ];
		} else {
			$numbered_list_circle_color = '#94cd4b';
		}

		// Grid Size

		if ( isset( $instance[ 'grid_size' ] ) ) {
			$grid_size = $instance[ 'grid_size' ];
		} else {
			$grid_size = 'medium';
		}

		// Thumbnail

		if ( isset( $instance[ 'thumbnail' ] ) ) {
			$thumbnail = $instance[ 'thumbnail' ];
		} else {
			$thumbnail = true;
		}

		// Show Category

		if ( isset( $instance[ 'show_category' ] ) ) {
			$show_category = $instance[ 'show_category' ];
		} else {
			$show_category = false;
		}

		// Show Author

		if ( isset( $instance[ 'show_author' ] ) ) {
			$show_author = $instance[ 'show_author' ];
		} else {
			$show_author = false;
		}

		// Show Post Date

		if ( isset( $instance[ 'show_post_date' ] ) ) {
			$show_post_date = $instance[ 'show_post_date' ];
		} else {
			$show_post_date = false;
		}

		// Show Views

		if ( isset( $instance[ 'show_views' ] ) ) {
			$show_views = $instance[ 'show_views' ];
		} else {
			$show_views = true;
		}

		// Show Comments

		if ( isset( $instance[ 'show_comments' ] ) ) {
			$show_comments = $instance[ 'show_comments' ];
		} else {
			$show_comments = true;
		}

		// Async

		if ( isset( $instance[ 'async' ] ) ) {
			$async = $instance[ 'async' ];
		} else {
			$async = true;
		}

		// Style Override

		if ( isset( $instance[ 'style_override' ] ) ) {
			$style_override = $instance[ 'style_override' ];
		} else {
			$style_override = true;
		}

		// Show Nav

		if ( isset( $instance[ 'show_nav' ] ) ) {
			$show_nav = $instance[ 'show_nav' ];
		} else {
			$show_nav = false;
		}

		// Check if there are custom post types

		$custom_post_types = get_post_types(array('public'   => true,'_builtin' => false));

		if (count($custom_post_types) == 0) {
			$custom_post_types = false;
		} else {
			$custom_post_types = true;
		}

		$id_prefix = $this->id . self::$prefix_dash;

		// View

		include(NNROBOTS_POPULAR_POSTS_PLUGIN_DIR . '/admin/views/widget-settings.php');

		// Styles

		wp_enqueue_style('wp-color-picker');
		wp_enqueue_style(self::$prefix_dash . 'selectize_css', plugins_url('admin/css/selectize.css', __FILE__));

		// Scripts

		wp_enqueue_script(self::$prefix_dash . 'widget_settings_js', plugins_url('admin/js/widget-settings.js', __FILE__), array( 'wp-color-picker' ), false, true );
		wp_enqueue_script(self::$prefix_dash . 'selectize_js', plugins_url('admin/js/selectize.min.js', __FILE__), array('jquery', 'jquery-ui-core', 'jquery-ui-sortable'));
		wp_localize_script(self::$prefix_dash . 'widget_settings_js', 'nnr_pp_widget_settings', array(
			'prefix'		=> self::$prefix_dash,
			'id_prefix'		=> $id_prefix
		));

	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = array();

		$instance['title']                        = ( ! empty( $new_instance['title'] ) ) ?
													sanitize_text_field( $new_instance['title'] ) : '';

		$instance['content']                      = ( ! empty( $new_instance['content'] ) ) ?
													sanitize_text_field( $new_instance['content'] ) : '';

		$instance['content_post_category']        = ( ! empty( $new_instance['content_post_category'] ) ) ?
													$new_instance['content_post_category'] : '';

		$instance['content_post_tag']             = ( ! empty( $new_instance['content_post_tag'] ) ) ?
													sanitize_text_field( $new_instance['content_post_tag'] ) : '';

		$instance['content_custom_post_type']     = ( ! empty( $new_instance['content_custom_post_type'] ) ) ?
													$new_instance['content_custom_post_type'] : '';

		$instance['popularity']                   = ( ! empty( $new_instance['popularity'] ) ) ?
													sanitize_text_field( $new_instance['popularity'] ) : '';

		$instance['popularity_time_range']        = ( ! empty( $new_instance['popularity_time_range'] ) ) ?
													sanitize_text_field( $new_instance['popularity_time_range'] ) : '';

		$instance['layout']                       = ( ! empty( $new_instance['layout'] ) ) ?
													sanitize_text_field( $new_instance['layout'] ) : '';

		$instance['fancy_colors']                 = ( ! empty( $new_instance['fancy_colors'] ) ) ?
													$new_instance['fancy_colors'] : '';

		$instance['color']                        = ( ! empty( $new_instance['color'] ) ) ?
													sanitize_text_field( $new_instance['color'] ) : '';

		$instance['text_color']                   = ( ! empty( $new_instance['text_color'] ) ) ?
													sanitize_text_field( $new_instance['text_color'] ) : '';

		$instance['num_of_posts']                 = ( ! empty( $new_instance['num_of_posts'] ) ) ?
													sanitize_text_field( $new_instance['num_of_posts'] ) : '';

		$instance['numbered_list_style']          = ( ! empty( $new_instance['numbered_list_style'] ) ) ?
													sanitize_text_field( $new_instance['numbered_list_style'] ) : '';

		$instance['numbered_list_color']          = ( ! empty( $new_instance['numbered_list_color'] ) ) ?
													sanitize_text_field( $new_instance['numbered_list_color'] ) : '';

		$instance['numbered_list_circle_color']   = ( ! empty( $new_instance['numbered_list_circle_color'] ) ) ?
													sanitize_text_field( $new_instance['numbered_list_circle_color'] ) : '';

		$instance['grid_size']                    = ( ! empty( $new_instance['grid_size'] ) ) ?
													sanitize_text_field( $new_instance['grid_size'] ) : '';

		$instance['thumbnail']                    = ( ! empty( $new_instance['thumbnail'] ) ) ?
													sanitize_text_field( $new_instance['thumbnail'] ) : '';

		$instance['show_category']                = ( ! empty( $new_instance['show_category'] ) ) ?
													sanitize_text_field( $new_instance['show_category'] ) : '';

		$instance['show_author']                  = ( ! empty( $new_instance['show_author'] ) ) ?
													sanitize_text_field( $new_instance['show_author'] ) : '';

		$instance['show_post_date']               = ( ! empty( $new_instance['show_post_date'] ) ) ?
													sanitize_text_field( $new_instance['show_post_date'] ) : '';

		$instance['show_views']                   = ( ! empty( $new_instance['show_views'] ) ) ?
													sanitize_text_field( $new_instance['show_views'] ) : '';

		$instance['show_comments']                = ( ! empty( $new_instance['show_comments'] ) ) ?
													sanitize_text_field( $new_instance['show_comments'] ) : '';

		$instance['async']                        = ( ! empty( $new_instance['async'] ) ) ?
													sanitize_text_field( $new_instance['async'] ) : '';

		$instance['style_override']               = ( ! empty( $new_instance['style_override'] ) ) ?
													sanitize_text_field( $new_instance['style_override'] ) : '';

		$instance['show_nav']                     = ( ! empty( $new_instance['show_nav'] ) ) ?
													sanitize_text_field( $new_instance['show_nav'] ) : '';

		return $instance;
	}

	/**
	 * Display popular posts based on list
	 *
	 * @access public
	 * @static
	 * @param mixed $instance
	 * @param mixed $posts
	 * @return void
	 */
	public static function display_popular_posts($instance, $shortcode = false) {

		$posts = self::getPopularPosts($instance);

		$classes = '';
		$id = '';

		if ($shortcode) {
			$id = $instance['id'];
			$classes .= ' ' . self::$prefix_dash . 'shortcode';
		}

		if (isset($instance[ 'meta_data' ]) && $instance[ 'meta_data' ]) {
			$classes .= ' ' . self::$prefix_dash . 'meta_data';
		}

		$content = '<div id="' . $id . '" class="' . $classes . '">';


		// List
		if (isset($instance['layout']) && $instance['layout'] == 'list') { $content .= '<ul class="' . self::$prefix_dash . 'list">'; }

		// Numbered List
		if (isset($instance['layout']) && $instance['layout'] == 'numbered_list') { $content .= '<ol class="' . self::$prefix_dash . 'numbered-list ' . self::$prefix_dash . 'numbered-list-' . $instance['numbered_list_style'] . '">'; }

		// Bar Graph
		if (isset($instance['layout']) && $instance['layout'] == 'bar_graph') { $content .= '<div class="' . self::$prefix_dash . 'bar-graph">';}

		// Line Graph
		if (isset($instance['layout']) && $instance['layout'] == 'line_graph') { $content .= '<div class="' . self::$prefix_dash . 'line-graph">';}

		// Image Tiles
		if (isset($instance['layout']) && $instance['layout'] == 'image_tile') { $content .= '<div class="' . self::$prefix_dash . 'image-tiles">';}

		// Thumbnail
		if (isset($instance['layout']) && $instance['layout'] == 'thumbnail') { $content .= '<ul class="' . self::$prefix_dash . 'thumbnail">'; }

		// Masonry
		if (isset($instance['layout']) && $instance['layout'] == 'masonry') { $content .= '<div class="' . self::$prefix_dash . 'masonry">';}

		// Grid
		if (isset($instance['layout']) && $instance['layout'] == 'grid') { $content .= '<div class="' . self::$prefix_dash . 'grid ' . self::$prefix_dash . 'grid-' . $instance['grid_size'] . '">';}

		// Slider
		if (isset($instance['layout']) && $instance['layout'] == 'slider') { $content .= '<div class="' . self::$prefix_dash . 'slider flexslider"><ul class="slides">';}

		// Fancy Colors
		if (isset($instance['layout']) && $instance['layout'] == 'fancy_colors') {$content .= '<div class="' . self::$prefix_dash . 'fancy-colors">';}

		// Determine Start and End Dates

		$dates                			= self::getStartEndDates($instance);
		$start_date           			= $dates[0];
		$end_date             			= $dates[1];
		$num_of_posts         			= 0;

		$temp_popular_posts        		= self::sortPopularPosts($posts, $start_date, $end_date, $instance[ 'popularity' ]);
		$popular_posts        			= $temp_popular_posts[0];
		$popular_posts_other            = $temp_popular_posts[1];
		$popular_posts_views_total  	= $temp_popular_posts[2];
		$popular_posts_comments_total  	= $temp_popular_posts[3];

		if ($instance[ 'popularity' ] == 'views') {
			$popular_posts_total = $popular_posts_views_total;
		} else {
			$popular_posts_total = $popular_posts_comments_total;
		}

		foreach ($popular_posts as $key => $popular_posts_count) {

			$temp_post_title = $posts[$key]->post_title;

			if (isset($instance['layout']) && $instance['layout'] == 'bar_graph' && strlen($temp_post_title) > 34) {
				$temp_post_title = substr($temp_post_title, 0, 33);
				$temp_post_title .= '&hellip;';
			}

			if ( ($instance[ 'popularity' ] == 'views' && $popular_posts_views_total < 1) || ($instance[ 'popularity' ] == 'comments' && $popular_posts_comments_total < 1) ) {
				return 'Hold on a second! No data yet.';
			}

			if ($num_of_posts >= $instance['num_of_posts']) {
				break;
			}

			// Post Title

			$post_title = '<a href="' . get_permalink($posts[$key]->ID) . '" class="' . self::$prefix_dash . 'title" title="' . $posts[$key]->post_title . '">' . $temp_post_title . '</a>';

			// Views / Comments

			$views = '';
			$comments = '';

			if ($instance[ 'popularity' ] == 'views') {

				if (isset($instance[ 'show_views' ]) && $instance[ 'show_views' ]) {
					$views = '<span class="' . self::$prefix_dash . 'views">' . self::displayViews($popular_posts_count) . '</span>';
				}

				if (isset($instance[ 'show_comments' ]) && $instance[ 'show_comments' ]) {
					$comments = '<span class="' . self::$prefix_dash . 'comments">' . self::displayComments($popular_posts_other[$key]) . '</span>';
				}

				$stats_data = '<div class="' . self::$prefix_dash . 'stats">' . $views . $comments . '</div>';
			} else {

				if (isset($instance[ 'show_views' ]) && $instance[ 'show_views' ]) {
					$views = '<span class="' . self::$prefix_dash . 'views">' . self::displayViews($popular_posts_other[$key]) . '</span>';
				}

				if (isset($instance[ 'show_comments' ]) && $instance[ 'show_comments' ]) {
					$comments = '<span class="' . self::$prefix_dash . 'comments">' . self::displayComments($popular_posts_count) . '</span>';
				}

				$stats_data = '<div class="' . self::$prefix_dash . 'stats">' . $comments . $views . '</div>';
			}

			// Post meta data

			$post_meta_data = '<div class="' . self::$prefix_dash . 'meta-data">';

			if ($instance['layout'] != 'masonry' && $instance['layout'] != 'grid') {

				// Category

				$category = '';

				if (isset($instance['show_category']) && $instance['show_category']) {
					$category 		= get_the_category( $posts[$key]->ID );
					$category_link 	= get_category_link( $category[0]->term_id );
					$category 		= '<a href="' . esc_url( $category_link ) . '" class="' . self::$prefix_dash . 'category" title="' . $category[0]->cat_name . '">' . $category[0]->cat_name . '</a>';
				}

				// Author

				$author = '';

				if (isset($instance['show_author']) && $instance['show_author']) {

					$author_data 	= get_userdata( $posts[$key]->post_author );
					$author 		= '<a href="' . get_author_posts_url( $posts[$key]->post_author ) . '" class="' . self::$prefix_dash . 'author" title="' . $author_data->user_nicename . '">' . $author_data->user_nicename . '</a>';
				}

				// Date

				$date = '';

				if (isset($instance['show_post_date']) && $instance['show_post_date']) {

					$display_date 	= date('M j, Y', strtotime($posts[$key]->post_date));
					$archive_year  	= date('Y', strtotime($posts[$key]->post_date));
					$archive_month 	= date('m', strtotime($posts[$key]->post_date));
					$archive_day   	= date('d', strtotime($posts[$key]->post_date));
					$date 			= '<a href="' .  get_day_link($archive_year, $archive_month, $archive_day) . '" class="' . self::$prefix_dash . 'date" title="' . $display_date . '">' . $display_date . '</a>';
				}

				$post_meta_data .= $category . $author . $date;
			}

			$post_meta_data .= '</div>';

			switch ( $instance['layout'] ) {

				case 'list':

					$content .= '<li>' . $post_title . $stats_data . $post_meta_data . '</li>';

				break;

				case 'numbered_list':

					$content .= '<li>';

					if ($instance['numbered_list_style'] != 'normal') {
						$content .= '<div class="' . self::$prefix_dash . 'numbered-list-number"></div><div class="' . self::$prefix_dash . 'numbered-list-title">';
					}

					$content .= $post_title . $stats_data . $post_meta_data;

					if ($instance['numbered_list_style'] != 'normal') {
						$content .= '</div>';
					}

					$content .= '</li>';

				break;

				case 'thumbnail':

					$content .= '<li><div class="' . self::$prefix_dash . 'thumbnail-image-container">' . get_the_post_thumbnail( $posts[$key]->ID, 'thumbnail', array('class'	=> self::$prefix_dash . 'thumnail-image')) . '</div><div class="' . self::$prefix_dash . 'thumbnail-data-container">' . $post_title . $stats_data . $post_meta_data . '</div></li>';

				break;

				case 'bar_graph':

					$content .= '<div class="' . self::$prefix_dash . 'bar-graph-color" style="width: ' . (10 + (floor(100 * ($popular_posts_count / $popular_posts_total)) * 0.9)) . '%;">' .
						$post_title . $stats_data . $post_meta_data .
					'</div>';

				break;

				case 'line_graph':

					$content .= '<div class="' . self::$prefix_dash . 'line-graph-item">
						<div class="' . self::$prefix_dash . 'line-graph-color" style="width: ' . (10 + (floor(100 * ($popular_posts_count / $popular_posts_total)) * 0.9)) . '%;"></div>
						<div class="' . self::$prefix_dash . 'line-graph-content">' .
							$post_title . $stats_data . $post_meta_data .
						'</div>
					</div>';

				break;

				case 'image_tile':

					$content .= '
					<div class="' . self::$prefix_dash . 'image-tile-item">' .
							get_the_post_thumbnail( $posts[$key]->ID, 'large') .
						'<div>' .
							$post_title .
						'</div>' .
						'<div class="' . self::$prefix_dash . 'image-tile-data">' .
							$stats_data .
							$post_meta_data .
						'</div>' .


					'</div>';

				break;

				case 'masonry':

					$image = wp_get_attachment_image_src(get_post_thumbnail_id($posts[$key]->ID), 'large');

					if ($image) {
						$content .= '<a class="' . self::$prefix_dash . 'title" href="' . get_permalink($posts[$key]->ID) . '" title="' . $posts[$key]->post_title . '"><img alt="' . $posts[$key]->post_title . '" src="' . $image[0] . '" /></a>';
					}

				break;

				case 'grid':

					if ($shortcode) {
						$image = wp_get_attachment_image_src(get_post_thumbnail_id($posts[$key]->ID), 'large');
					} else {
						$image = wp_get_attachment_image_src(get_post_thumbnail_id($posts[$key]->ID));
					}

					if ($image) {
						$content .= '<div class="' . self::$prefix_dash . 'grid-item"><a class="' . self::$prefix_dash . 'title" href="' . get_permalink($posts[$key]->ID) . '" title="' . $posts[$key]->post_title . '" style="background-image: url(' . $image[0] . ');"></a></div>';
					}

				break;

				case 'slider':

					$image = wp_get_attachment_image_src(get_post_thumbnail_id($posts[$key]->ID), 'large');

					if ($image) {
						$content .= '<li><img alt="' . $posts[$key]->post_title . '" src="' . $image[0] . '" />';
						$content .= '<div class="image-caption">
							<a class="' . self::$prefix_dash . 'title" href="' . get_permalink($posts[$key]->ID) . '">' . $posts[$key]->post_title . '</a>' .
							$stats_data .
							$post_meta_data .
						'</div>';
						$content .= '</li>';
					}

				break;

				case 'fancy_colors':

					$fancy_colors = explode(',', $instance['fancy_colors']);

					if ($num_of_posts >= count($fancy_colors)) {
						$color = $fancy_colors[count($fancy_colors) - 1];
					} else {
						$color = $fancy_colors[$num_of_posts];
					}

					$content .= '<div class="' . self::$prefix_dash . 'fancy-colors-content" style="background-color: ' . $color . ';">' .
						$post_title . $stats_data . $post_meta_data .
					'</div>';

				break;
			}

			$num_of_posts++;
		}

		// List
		if (isset($instance['layout']) && $instance['layout'] == 'list') { $content .= '</ul>'; }

		// Numbered List
		if (isset($instance['layout']) && $instance['layout'] == 'numbered_list') { $content .= '</ol>'; }

		// Bar Graph
		if (isset($instance['layout']) && $instance['layout'] == 'bar_graph') { $content .= '</div>';}

		// Line Graph
		if (isset($instance['layout']) && $instance['layout'] == 'line_graph') { $content .= '</div>';}

		// Image Tiles
		if (isset($instance['layout']) && $instance['layout'] == 'image_tile') { $content .= '</div>';}

		// Thumbnail
		if (isset($instance['layout']) && $instance['layout'] == 'thumbnail') { $content .= '</ul>'; }

		// Masonry
		if (isset($instance['layout']) && $instance['layout'] == 'masonry') { $content .= '</div>';}

		// Grid
		if (isset($instance['layout']) && $instance['layout'] == 'grid') { $content .= '</div>';}

		// Slider
		if (isset($instance['layout']) && $instance['layout'] == 'slider') { $content .= '</ul></div>';}

		// Fancy Colors
		if (isset($instance['layout']) && $instance['layout'] == 'fancy_colors') {$content .= '</div>';}

		$content .= '</div>';

		return $content;
	}

	/**
	 * Gets all posts based on user widget settings
	 *
	 * @access public
	 * @static
	 * @param mixed $instance
	 * @return array $posts
	 */
	public static function getPopularPosts($instance) {

		// Determine what posts to get

		$post_args = array(
			'posts_per_page'   => '-1',
			'offset'           => 0,
			'post_status'      => 'publish'
		);

		// Category

		if (isset($instance[ 'content' ]) && $instance[ 'content' ] == 'category') {

			$cat_list = '';

			foreach($instance['content_post_category'] as $cat) {
				$cat_list .= $cat . ',';
			}

			$post_args['category'] = $cat_list;
		}

		// Tag

		if (isset($instance[ 'content' ]) && $instance[ 'content' ] == 'tag' && $instance[ 'content_post_tag' ] != '') {
			$post_args['tag'] = $instance[ 'content_post_tag' ];
		}

		// Custom Post Type

		if (isset($instance[ 'content' ]) && $instance[ 'content' ] == 'custom_post_type') {
			$post_args['post_type'] = $instance[ 'content_custom_post_type' ];
		}

		// All Post Types

		if (isset($instance[ 'content' ]) && $instance[ 'content' ] == 'all_post_types') {
			$post_args['post_type'] = 'any';
		}

		// Only Posts

		if (isset($instance[ 'content' ]) && $instance[ 'content' ] == 'only_posts') {
			$post_args['post_type'] = 'post';
		}

		// Only Pages

		if (isset($instance[ 'content' ]) && $instance[ 'content' ] == 'only_pages') {
			$post_args['post_type'] = 'page';
		}

		$posts = get_posts($post_args);

		return $posts;
	}

	/**
	 * Get start and end dates
	 *
	 * @access public
	 * @param mixed $instance
	 * @return array($start_date, $end_date)
	 */
	public static function getStartEndDates($instance) {

		// Determine Start and End Dates

		$end_date = date("Y-m-d");

		if (isset($instance[ 'popularity_time_range' ]) && $instance[ 'popularity_time_range' ] == '1_day') {
			$start_date = date("Y-m-d", mktime(0, 0, 0, date("n"), date("j") - 1, date("Y")));
		} else if (isset($instance[ 'popularity_time_range' ]) && $instance[ 'popularity_time_range' ] == '7_days') {
			$start_date = date("Y-m-d", mktime(0, 0, 0, date("n"), date("j") - 7, date("Y")));
		} else if (isset($instance[ 'popularity_time_range' ]) && $instance[ 'popularity_time_range' ] == '30_days') {
			$start_date = date("Y-m-d", mktime(0, 0, 0, date("n"), date("j") - 30, date("Y")));
		} else if (isset($instance[ 'popularity_time_range' ]) && $instance[ 'popularity_time_range' ] == '1_year') {
			$start_date = date("Y-m-d", mktime(0, 0, 0, date("n"), date("j"), date("Y") - 1));
		} else {
			$start_date = null;
		}

		return array($start_date, $end_date);
	}

	/**
	 * Get a sorted array of posts based on either views or comments
	 *
	 * @access public
	 * @static
	 * @param mixed $posts
	 * @param mixed $start_date
	 * @param mixed $end_date
	 * @param mixed $popularity
	 * @return void
	 */
	static function sortPopularPosts($posts, $start_date, $end_date, $popularity) {

		$popular_posts            = array();
		$temp_popular_posts_other = array();
		$views_max                = 0;
		$comments_max             = 0;
		$i                        = 0;

		foreach ($posts as $post) {

			// Get all views by post

			$temp_views = NNRobots_Popular_Posts_DB::getViews(array($post), $start_date, $end_date);
			$temp_view_total = 0;

			// Calculate total number of views

			foreach ($temp_views as $temp_view) {
				$temp_view_total += $temp_view->views;
			}

			// Set new max if this posts views are greater than current max views

			if ($temp_view_total > $views_max) {
				$views_max = $temp_view_total;
			}

			// Get all comments by post

			$temp_comments = NNRobots_Popular_Posts_DB::getComments(array($post), $start_date, $end_date);

			// Set new max if this posts comments are greater than current max comments

			if (count($temp_comments) > $comments_max) {
				$comments_max = count($temp_comments);
			}

			// Add the views and comments number to the respective arrays based off popularity filter

			if ($popularity == 'views') {
				$popular_posts[$i] 				= $temp_view_total;
				$temp_popular_posts_other[$i] 	= count($temp_comments);
			} else {
				$popular_posts[$i] 				= count($temp_comments);
				$temp_popular_posts_other[$i] 	= $temp_view_total;
			}

			$i++;
		}

		// Use the associative array sort based on value to sort array

		arsort($popular_posts);

		$popular_posts_other = array();

		foreach ($popular_posts as $key => $value) {

			$popular_posts_other[$key] = $temp_popular_posts_other[$key];
		}

		return array($popular_posts, $popular_posts_other, $views_max, $comments_max);

	}

	/**
	 * Display Comments
	 *
	 * @access public
	 * @static
	 * @param mixed $comment_count
	 * @return void
	 */
	public static function displayComments($comment_count) {
		if ($comment_count == 0) {
			$print_comments = __('No Comments', self::$text_domain);
		} else if ($comment_count == 1) {
			$print_comments = __('1 Comment', self::$text_domain);
		} else if ($comment_count > 1) {
			$print_comments = sprintf(__('%d Comments', self::$text_domain), $comment_count);
		}

		return $print_comments;
	}

	/**
	 * Display Views
	 *
	 * @access public
	 * @static
	 * @param mixed $view_count
	 * @return void
	 */
	public static function displayViews($view_count) {
		if ($view_count == 0) {
			$print_views = __('No Views', self::$text_domain);
		} else if ($view_count == 1) {
			$print_views = __('1 View', self::$text_domain);
		} else if ($view_count > 1) {
			$print_views = sprintf(__('%d Views', self::$text_domain), $view_count);
		}

		return $print_views;
	}

} // class NNRobots_Popular_Posts_Widget